#include "StdAfx.h"
#include "speech_rec.h"

speech_rec::speech_rec():stream(NULL)
{
}

speech_rec::speech_rec(string *keywords, int nbr_keywords, float seconds, int Speech_det, int Sample_rate, int frames_per_buffer, param* p):stream(NULL)
{
	init(keywords, nbr_keywords, seconds, Speech_det, Sample_rate, frames_per_buffer, p);
}

speech_rec::~speech_rec(void)
{
}

bool speech_rec::init(string *keywords, int nbr_keywords, float seconds, int Speech_det, int Sample_rate, int frames_per_buffer, param* p) //launch functions to create the recognition grammar and initialise portaudio and ear
{
	bool b=create_gram(keywords, nbr_keywords,"sampa_beep.txt","Transcriptions.fsg");
	//b=b&&initialise_ear(Speech_det);
	b=b&&set_rec_ready(seconds,Speech_det,Sample_rate,frames_per_buffer,p);
	return b;
}

bool speech_rec::isempty(const char *str) //check if empty
{
	while (*str != '\0')
	{
		if (isalnum(*str))
			return false;
		str++;
	}
	return true;
} 



string BG_StringToUpper (const string sBGString) //converts to UPPERCASE (need for building the phonetic transcription)
{
	char cChar;
	string sUpper = "";
	unsigned int iIndex = 0;

	while (iIndex < sBGString.length()){
		cChar = sBGString [iIndex];

		// if cChar between "a-z", transform into "A-Z"
		if (cChar >= 'a' && cChar <= 'z')
		cChar = cChar - 32;

		sUpper = sUpper + cChar;

		iIndex++;
	}

	return sUpper;
}



bool speech_rec::set_rec_ready(float seconds, int Speech_det,  int Sample_rate, int frames_per_buffer, param* p) //launch functions to initialise portaudio and ear
{
  bool b=initialise_ear(Speech_det);
  if(b)
	{
	return initialise_pa(seconds, Sample_rate,frames_per_buffer,p);
	}
	else 
	{
	return false;
	}
}

bool speech_rec::initialise_ear(int speech_det) //initialises ear
{
	unsigned int length; 
	//EarReturnCode rc;
	ear=EarOpen();
	mlp=EarModels("eng16lff.234-1000-045.mlp");
	app=MapBinaryFileToMem("Transcriptions.app",&length);
	
	CHECK(EarSetModels(ear,mlp));                            // load acoustic model
	CHECK(EarReadVocabulary(ear, app));                      // load grammatical model
	CHECK(EarSetSetting(ear,EarBeamWidth,250));			     // apply some pruning
	CHECK(EarSetSetting(ear,EarWordEntrancePenalty,20));	 // set word  entrance penalty to 10	      
	CHECK(EarSetSetting(ear,EarUseSpeechDetector,speech_det));	     // 1 means Yes
	CHECK(EarSetSetting(ear,EarSdMinimumSpeech,0.1));	     // require at least 300 ms of speech
	CHECK(EarSetSetting(ear,EarSdMaximumSilence,0.3));	     // allow up to 300 ms of silence inside an utterance
	CHECK(EarSetSetting(ear,EarSdSungIndicatorThreshold,10)); // vad setup
	CHECK(EarSetSetting(ear,EarSdSungSpeechThreshold,10));   // vad setup
	CHECK(EarSetSetting(ear,EarGarbage,0.1));			     // use a medium garbage model
	CHECK(EarSetSetting(ear,EarNBest,1));				     // get the 1 best recognition results
	

	return true;
}


static int recordCallback( const void *inputBuffer, void *outputBuffer,
 	                           unsigned long framesPerBuffer,
 	                           const PaStreamCallbackTimeInfo* timeInfo,
 	                           PaStreamCallbackFlags statusFlags,
 	                           void *userData )
 	{
 	    param *data = (param*)userData;
 	    const SAMPLE *rptr = (const SAMPLE*)inputBuffer;
 	    SAMPLE *wptr = &data->recordedSamples[data->frameIndex * NUM_CHANNELS];
 	    long framesToCalc;
 	    long i;
 	    int finished;
 	    unsigned long framesLeft = data->maxFrameIndex - data->frameIndex;
 	
 	    (void) outputBuffer; /* Prevent unused variable warnings. */
 	    (void) timeInfo;
 	    (void) statusFlags;
 	    (void) userData;
 	
 	    if( framesLeft < framesPerBuffer )
 	    {
 	        framesToCalc = framesLeft;
 	        finished = paComplete;
 	    }
 	    else
 	    {
 	        framesToCalc = framesPerBuffer;
 	        finished = paContinue;
 	    }
 	
 	    if( inputBuffer == NULL )
 	    {
 	        for( i=0; i<framesToCalc; i++ )
 	        {
 	            *wptr++ = SAMPLE_SILENCE;  /* left */
 	            if( NUM_CHANNELS == 2 ) *wptr++ = SAMPLE_SILENCE;  /* right */
 	        }
 	    }
 	    else
		{
 	        for( i=0; i<framesToCalc; i++ )
 	        {
 	            *wptr++ = *rptr++;  /* left */
 	            if( NUM_CHANNELS == 2 ) *wptr++ = *rptr++;  /* right */
 	        }
	    }
 	    data->frameIndex += framesToCalc;
	    return finished;
 	}

/*static int piloteCallback( const void *inputBuffer, void *outputBuffer,    //Callback function for portaudio: this function is called each time an audio buffer is filled. It sends this buffer to ear for recognition
                           unsigned long framesPerBuffer,
                           const PaStreamCallbackTimeInfo* timeInfo,
                           PaStreamCallbackFlags statusFlags,
                           void *userData )
{
	//int e;
	int f=(int)framesPerBuffer;
	


	param* p=(param*)userData;
	LPEAR e=p->ear;
	EarReturnCode rc;
	rc=EarRecognizeBuffer(e,inputBuffer,f);
	p->compteur++;
	//printf("\n%d",p->compteur);
	return 0; // 0=paContinue: permet de continuer le processus
}*/


bool speech_rec::initialise_pa(float seconds, int Sample_rate, int frames_per_buffer,param* p)  // initialises portaudio
{
	PaError err;
	PaStreamParameters inputParameters;
	int numSamples, numBytes,totalFrames,i;

	//PaStreamParameters outputParameters;
	err = Pa_Initialize();
	if( err != paNoError )
	{
    printf(  "PortAudio error: %s\n", Pa_GetErrorText( err ) );
	return false;
	}

	p->maxFrameIndex = totalFrames = (int)seconds * Sample_rate;//NUM_SECONDS * SAMPLE_RATE; /* Record for a few seconds. */
 	p->frameIndex = 0;
 	numSamples = totalFrames * NUM_CHANNELS;//NUM_CHANNELS;
 	numBytes = numSamples * sizeof(SAMPLE);
 	p->recordedSamples = (SAMPLE *) malloc( numBytes ); /* From now on, recordedSamples is initialised. */
 	if(p->recordedSamples == NULL )
		{
 	        printf("Could not allocate record array.\n");
 	        goto done;
 	    } 	    
	for( i=0; i<numSamples; i++ ) p->recordedSamples[i] = 0;


	inputParameters.channelCount = 1;
    inputParameters.device = Pa_GetDefaultInputDevice();
    inputParameters.hostApiSpecificStreamInfo = NULL;
    inputParameters.sampleFormat = paInt16;
    inputParameters.suggestedLatency = Pa_GetDeviceInfo(Pa_GetDefaultInputDevice())->defaultLowInputLatency ;

	/*outputParameters.channelCount = 2;
    outputParameters.device = Pa_GetDefaultOutputDevice();
    outputParameters.hostApiSpecificStreamInfo = NULL;
    outputParameters.sampleFormat = paInt16;
    outputParameters.suggestedLatency = Pa_GetDeviceInfo(Pa_GetDefaultOutputDevice())->defaultLowInputLatency ;*/

	//*p=NULL;
	//param* k=*p;
	/*p->ear=ear;
	p->speech_det=speech_det;
	p->compteur=0;*/
	

	err = Pa_OpenStream( &stream,
                         &inputParameters,          /* input */
						 NULL,
						 Sample_rate,
						 frames_per_buffer,
                        /* paFramesPerBufferUnspecified, */       /* frames per buffer, i.e. the number
                                                   of sample frames that PortAudio will
                                                   request from the callback. Many apps
                                                   may want to use
                                                   paFramesPerBufferUnspecified, which
                                                   tells PortAudio to pick the best,
                                                   possibly changing, buffer size.*/
					      paNoFlag,
						  recordCallback, /* this is your callback function */
                          p); /*This is a pointer that will be passed to
                                                   your callback*/
    if( err != paNoError )
	{
    printf(  "PortAudio error: %s\n", Pa_GetErrorText( err ) );
	return false;
	}

	return true;

	done:
	    Pa_Terminate();
	    if( p->recordedSamples )       /* Sure it is NULL or valid. */
 	        free( p->recordedSamples );
 	    if( err != paNoError )
 	    {
 	        fprintf( stderr, "An error occured while using the portaudio stream\n" );
 	        fprintf( stderr, "Error number: %d\n", err );
 	        fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );
 	        err = 1;          /* Always return 0 or 1, but no other return codes. */
 	    }
		return false;
}

int speech_rec::close_ear()  //nothing to add to the function name...
{
	//EarReturnCode rc;
	//close ear
	CHECK(EarModelsClose(mlp));
	CHECK(EarClose(ear));
	return 0;
}



char* speech_rec::MapBinaryFileToMem(const char *fname, unsigned int *bufsize)
{
  FILE* fin;
  unsigned long pos;
  char* buffer;
  
  if (!(fin = fopen(fname, "rb")))
  {
	  fprintf(stderr, "Cannot open file %s.\n", fname);
	  return NULL;
  }
  fseek(fin,0,SEEK_END);
  pos = ftell(fin);
  fseek(fin,0,SEEK_SET);
  *bufsize = pos;
  buffer = (char*) malloc ((pos + 1) * sizeof(char));
  if (buffer == NULL)
  {
	  fprintf(stderr, "Cannot allocate memory for %s.\n", fname);
	  return NULL;
  }
  memset(buffer,0, pos + 1);
  fread(buffer, 1, pos, fin);
  fclose(fin);
  return buffer;
} 





results speech_rec::do_rec(float seconds, int sample_rate, int frames_per_buffer,param *p) //launch audio acquisition for "seconds" seconds, then calls the recognition results
{
	PaError err = Pa_StartStream(stream);

	//printf("Now recording!!\n"); fflush(stdout);
 	
 	    while( ( err = Pa_IsStreamActive( stream ) ) == 1 )
	    {
	        Pa_Sleep(1000);
 	        //printf("index = %d\n", p->frameIndex ); fflush(stdout);
 	    }
	    //if( err < 0 ) return "error in audio acquisition";

	rc=EarRecognizeBuffer(ear,p->recordedSamples,p->maxFrameIndex);
	close_pa();
	initialise_pa(seconds, sample_rate,frames_per_buffer,p);
	/*while(p->compteur*frames_per_buffer/sample_rate<seconds)
		{

		}*/
	//err = Pa_StopStream(stream);

	results c=terminate_recognition();
	return c;
}

results speech_rec::terminate_recognition()
{
	rc=EarRecognizeBuffer(ear,NULL,0);                   // terminate recognition
	results c=give_recognition_results();
	return c;
}

results speech_rec::give_recognition_results()   //gives back (and display) the result of the speech recognition
{
	int rec_count;
	const char ***word;
	results c;
	float **confidence;
	//EarReturnCode rc;
	rec_count=EarRecognizedWordCount(ear,0);

	//if(rec_count) printf("\nRec count: %d\n",rec_count);
	rc=EarGetNBestStrings(ear,&word,NULL,&confidence,NULL);
	for (int i = 0; i < rec_count; i++)
			{
				//printf("\nMot %d: ", i+1);
				if (!isempty(word[0][i]))
					{
						printf("\n word:%s\tconfidence:%8.4f",word[0][i],confidence[0][i]);
						c.word=word[0][i];
						c.confidence=confidence[0][i];
						return c;
					}	
			}
	c.confidence=-1;
	c.word="kpfgd";
	return c;
}






void speech_rec::close_pa()  // close portaudio
{
  PaError err = Pa_CloseStream(stream);
	if( err != paNoError )
    printf(  "PortAudio error: %s\n", Pa_GetErrorText( err ) );

	err = Pa_Terminate();
	if( err != paNoError )
    printf(  "PortAudio error: %s\n", Pa_GetErrorText( err ) );
}






bool speech_rec::create_gram(string *keywords, int nbr_keywords,char* dictname, char* out_name)  //creates the grammar file from the keywords
{
	//string *s=getOptions();
	string temp, dict, go;
	const char *d, *k;
	int a, i;

	ifstream phonedict;
	ofstream phonetransc;


	phonetransc.open(out_name);

	if(phonetransc.is_open()){
		go="#!FSG";
		phonetransc << go << endl;
		go="\n";
		phonetransc << go << endl;
		go="<EXPRESSION> = alt(";
		phonetransc << go << endl;
	  
		for(i=0;i<nbr_keywords;i++){
			temp=BG_StringToUpper(keywords[i]);
			phonetransc << temp << endl;
		}

		go=")alt;";
		  phonetransc << go << endl;
		  go="\n";
		  phonetransc << go << endl;
		  go="<START> = seq(";
		  /*phonestransc << go << endl;
		  go="\t\trep(";*/
		  phonetransc << go << endl;
		  go="\t\t\t<EXPRESSION>";
		  /*phonestransc << go << endl;
		  go="\t\t)rep";*/
		  phonetransc << go << endl;
		  go="\t)seq";
		  phonetransc << go << endl;
		  go="\n";
		  phonetransc << go << endl;
		  go="#!Transcriptions";
		  phonetransc << go << endl;


		  for(int q=0;q<nbr_keywords;q++){
				a=0;
				temp=BG_StringToUpper(keywords[q]);
				k=temp.c_str();
			  /*for (n=0;n<strlen(k);n++)
				{
					k[n]=(char)toupper(k[n]);
				}*/
			  //strcpy(k,g);
			  
				//cout << keyword << endl;
				phonedict.open(dictname);
			   if (phonedict.is_open()){
					i=1;
					while (! phonedict.eof()&& i &&!a ){
						getline (phonedict,dict);
						d=dict.c_str();
						i=strncmp(k,d,strlen(k));
						while(!i){
							if(d[strlen(k)]==' '||d[strlen(k)]=='('){ 
							  phonetransc << dict << endl;
							  //cout << d[strlen(k)] << endl;
							  //i=1; // pour aller voir la ligne d'aprs
							  a++;
							}
							getline (phonedict,dict);
							d=dict.c_str();
							i=strncmp(k,d,strlen(k));
						}
					}
					phonedict.close();
			   }else{
				   cout << "Unable to open file";
				   return false;
			   }
		}
		phonetransc.close();
		system("strut.exe compile-asr setup=Transcriptions.app.stp");
	}else{
		cout << "Unable to open file";
		return false;
	}

  //phonedict.close();
  //keywords.close();


  return true;
}