// FaceDetection.cpp: implementation of the CFaceDetection class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "FaceDetection.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFaceDetection::CFaceDetection(){
	const char* cascade_name = "./Data/haarcascade_frontalface_default.xml";
	//const char* cascade_name = "./Data/haarcascade_upperbody.xml";
    cascade = (CvHaarClassifierCascade*)cvLoad( cascade_name, 0, 0, 0 );
    if( !cascade ){fprintf( stderr, "ERROR: Could not load classifier cascade\n" );}
	storage = cvCreateMemStorage(0);
}

CFaceDetection::~CFaceDetection(){}

bool CFaceDetection::trackFace( IplImage* img , CvRect * head,CSkinMask * headsResearchMask){
	CvRect zone;
	zone.x=head->x;
	zone.y=head->y;
	zone.height=head->height;
	zone.width=head->width;
	CCommun::scaleCvRect(&zone,2,2);
	if(zone.x<0){zone.x=0;}
	if((zone.x+zone.width)>img->width){zone.width=img->width-zone.x-1;}
	if(zone.y<0){zone.y=0;}
	if((zone.y+zone.height)>img->height){zone.height=img->height-zone.y-1;}

	bool trouve=false;
	if(zone.height>0&&zone.width>0){
	//printf("zone.x=%d zone.y=%d	zone.height=%d zone.width=%d\n",zone.x,zone.y,zone.height,zone.width);
		CvSeq* faces=0;
		IplImage* gray = cvCreateImage( cvSize(img->width,img->height), 8, 1 );
		IplImage* small_img = cvCreateImage( cvSize(img->width,img->height),8, 1 );

		cvCvtColor( img, gray, CV_BGR2GRAY );
		cvResize( gray, small_img, CV_INTER_LINEAR );
		cvEqualizeHist( small_img, small_img );
		cvClearMemStorage( storage );

		cvSetImageROI(small_img, zone );
		if( cascade ){
			faces = cvHaarDetectObjects( small_img, cascade, storage,
												1.1, 2, CV_HAAR_DO_CANNY_PRUNING,
												cvSize(30, 30) );
		}
		cvResetImageROI(small_img);
		if(gray){cvReleaseImage(&gray);}
		if(small_img){cvReleaseImage(&small_img);}
		int dMin=0;
		int imin=-1;
		for(int i = 0; i < faces->total; i++ ){
			CvRect* tmp_head=(CvRect*)cvGetSeqElem(faces,i);
			//if(tmp_head->height<head->height*2&&tmp_head->height>head->height/2){
			int dx=tmp_head->x-head->x;if(dx<0){dx=-dx;}
			int dy=tmp_head->y-head->y;if(dy<0){dy=-dy;}
			int dw=tmp_head->width-head->width;if(dw<0){dw=-dw;}
			int dh=tmp_head->height-head->height;if(dh<0){dh=-dh;}
			//if(dx<head->width/2&&dy<head->height/2&&headsResearchMask->testMaskZone(30,*tmp_head)){
			if(dh<head->height/5&&dw<head->width/5){
				if(imin==-1){
					dMin=(tmp_head->height-head->height)*(tmp_head->height-head->height)+(tmp_head->width-head->width)*(tmp_head->width-head->width);
					imin=0;
				}else{
					int dCurrent=(tmp_head->height-head->height)*(tmp_head->height-head->height)+(tmp_head->width-head->width)*(tmp_head->width-head->width);
					if(dCurrent<dMin){
						imin=i;
						dMin=dCurrent;
					}
				}
			}
		}
		if(imin!=-1){
			CvRect* tmp_head=(CvRect*)cvGetSeqElem(faces,imin);
			head->x=tmp_head->x+zone.x;
			head->y=tmp_head->y+zone.y;
			head->height=tmp_head->height;
			head->width=tmp_head->width;
			CPeopleWithHands::scaleHead(head);
			trouve=true;
		}
	}
	return trouve;
}


bool CFaceDetection::trackFace( IplImage* img , CvRect * head){
	CvRect zone;
	zone.x=head->x;
	zone.y=head->y;
	zone.height=head->height;
	zone.width=head->width;
	CCommun::scaleCvRect(&zone,2,2);
	if(zone.x<0){zone.x=0;}
	if((zone.x+zone.width)>img->width){zone.width=img->width-zone.x-1;}
	if(zone.y<0){zone.y=0;}
	if((zone.y+zone.height)>img->height){zone.height=img->height-zone.y-1;}

	bool trouve=false;
	if(zone.height>0&&zone.width>0){
	//printf("zone.x=%d zone.y=%d	zone.height=%d zone.width=%d\n",zone.x,zone.y,zone.height,zone.width);
		CvSeq* faces=0;
		IplImage* gray = cvCreateImage( cvSize(img->width,img->height), 8, 1 );
		IplImage* small_img = cvCreateImage( cvSize(img->width,img->height),8, 1 );

		cvCvtColor( img, gray, CV_BGR2GRAY );
		cvResize( gray, small_img, CV_INTER_LINEAR );
		cvEqualizeHist( small_img, small_img );
		cvClearMemStorage( storage );

		cvSetImageROI(small_img, zone );
		if( cascade ){
			faces = cvHaarDetectObjects( small_img, cascade, storage,
												1.1, 2, CV_HAAR_DO_CANNY_PRUNING,
												cvSize(30, 30) );
		}
		cvResetImageROI(small_img);
		if(gray){cvReleaseImage(&gray);}
		if(small_img){cvReleaseImage(&small_img);}
		int dMin=0;
		int imin=-1;
		for(int i = 0; i < faces->total; i++ ){
			CvRect* tmp_head=(CvRect*)cvGetSeqElem(faces,i);
			//if(tmp_head->height<head->height*2&&tmp_head->height>head->height/2){
			int dx=tmp_head->x-head->x;if(dx<0){dx=-dx;}
			int dy=tmp_head->y-head->y;if(dy<0){dy=-dy;}
			if(dx<head->width/2&&dy<head->height/2){
				if(imin==-1){
					dMin=(tmp_head->height-head->height)*(tmp_head->height-head->height)+(tmp_head->width-head->width)*(tmp_head->width-head->width);
					imin=0;
				}else{
					int dCurrent=(tmp_head->height-head->height)*(tmp_head->height-head->height)+(tmp_head->width-head->width)*(tmp_head->width-head->width);
					if(dCurrent<dMin){
						imin=i;
						dMin=dCurrent;
					}
				}
			}
		}
		if(imin!=-1){
			CvRect* tmp_head=(CvRect*)cvGetSeqElem(faces,imin);
			head->x=tmp_head->x+zone.x;
			head->y=tmp_head->y+zone.y;
			head->height=tmp_head->height;
			head->width=tmp_head->width;
			CPeopleWithHands::scaleHead(head);
			trouve=true;
		}
	}
	return trouve;
}

bool CFaceDetection::trackFaceWithSkin(COpenCvImage * skinMaskOld,COpenCvImage * skinMask, CvRect * head){
	CvPoint pOld=skinMaskOld->getAverageCvPoint(head);
	CvPoint pNew=skinMask->getAverageCvPoint(head);
	if(pOld.x!=-1&&pNew.x!=-1){
		head->x=head->x+pNew.x-pOld.x;
		head->y=head->y+pNew.y-pOld.y;
		return true;
	}else{
		return false;
	}
}

CvSeq * CFaceDetection::detectFace( IplImage* img ){

	CvSeq* faces=0;
    IplImage* gray = cvCreateImage( cvSize(img->width,img->height), 8, 1 );
    IplImage* small_img = cvCreateImage( cvSize(img->width,img->height),8, 1 );

    cvCvtColor( img, gray, CV_BGR2GRAY );
    cvResize( gray, small_img, CV_INTER_LINEAR );
    cvEqualizeHist( small_img, small_img );
    cvClearMemStorage( storage );

    if( cascade ){
        faces = cvHaarDetectObjects( small_img, cascade, storage,
                                            1.1, 2, 0/*CV_HAAR_DO_CANNY_PRUNING*/,
                                            cvSize(30, 30) );
    }
	if(gray){cvReleaseImage(&gray);}
	if(small_img){cvReleaseImage(&small_img);}
	return faces;
}