// OpenCvImage.cpp: implementation of the COpenCvImage class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "OpenCvImage.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

int COpenCvImage::cptNamedWindow=0;

COpenCvImage::COpenCvImage(CvSize size):namedWindows(0),im(0){
	im=cvCreateImage(size,8,3);
	initPixelPos();
}

COpenCvImage::COpenCvImage(IplImage* original):namedWindows(0),im(0){
	setImage(original);
}

COpenCvImage::COpenCvImage(COpenCvImage * original):namedWindows(0){
	im=cvCreateImage(original->size(), IPL_DEPTH_8U ,3);
	if( original->im->origin == IPL_ORIGIN_TL ){
		cvCopy( original->im, im, 0 );
	}else{
		cvFlip( original->im, im, 0 );
	}
	initPixelPos();
}

COpenCvImage::~COpenCvImage(){
	if(im){cvReleaseImage(&im);}
	if(namedWindows){cvDestroyWindow(namedWindows);}
}

void COpenCvImage::setImage(IplImage* original){
	im=original;
	initPixelPos();
}

void COpenCvImage::initPixelPos(){
	CV_INIT_PIXEL_POS(pos,(unsigned char *) im->imageData,im->widthStep,cvGetSize(im),0,0,im->origin);
}

uchar * COpenCvImage::cvMoveTo(int x,int y){
	return CV_MOVE_TO(pos,x,y,3);
}

CvPoint COpenCvImage::getAverageCvPoint(CvRect * maskZone){
	uchar * ptr_frame;
	CvPoint res;
	int minx = maskZone->x;
	int maxx = maskZone->x+maskZone->width;
	int miny = maskZone->y;
	int maxy = maskZone->y+maskZone->height;
	if(minx<0){minx=0;}
	if(maxx>width()){maxx=width();}
	if(miny<0){miny=0;}
	if(maxy>height()){maxy=height();}
	int mx=0,my=0,cpt=0;

	for(int y=miny;y<maxy; y++){
		for (int x=minx; x<maxx; x++){
			ptr_frame = cvMoveTo(x,y);
			if(ptr_frame[0]==255){
				cpt++;
				my+=y;
				mx+=x;
			}
		}
	}
	if(cpt!=0){
		res.x=(int)(mx/cpt);
		res.y=(int)(my/cpt);
	}else{
		res.x=-1;
		res.y=-1;
	}
	return res;
}
/*
bool COpenCvImage::testZone(double pourcentage,int minx,int miny,int maxx,int maxy){
	CvPixelPosition8u pos_im;
	uchar * ptr_im;
	int x=0,y=0;
	int nbrPixelOn=0;
	int nbrPixel=0;

	if(minx<0){minx=0;}
	if(maxx>im->width){maxx=im->width;}
	if(miny<0){miny=0;}
	if(maxy>im->height){maxy=im->height;}

	CV_INIT_PIXEL_POS(pos_im,(unsigned char *) im->imageData,im->widthStep,cvGetSize(im),x,y,im->origin);

	for( y=miny;y<maxy; y++){
		for ( x=minx; x<maxx; x++){
			ptr_im = CV_MOVE_TO(pos_im,x,y,3);
			if( ptr_im[0] == 255 ){
				nbrPixelOn++;
			}
			nbrPixel++;
		}
	}
	return pourcentage/100.0<(double)nbrPixelOn/(double)(nbrPixel);
}*/

void COpenCvImage::removeCvRect(CvRect * r){
	fillZone(r->x,r->x+r->width,r->y,r->y+r->height,0);
}


void COpenCvImage::reset(){
	fillZone(0,im->width,0,im->height,0);
}

void COpenCvImage::fillZone(CvRect r,unsigned char value){
	fillZone(r.x,r.x+r.width,r.y,r.y+r.height,value);
}

void COpenCvImage::fillZone(int minx,int maxx,int miny,int maxy,unsigned char value){
	CvPixelPosition8u pos_im;
	uchar * ptr_im;
	int x=0,y=0;

	if(minx<0){minx=0;}
	if(maxx>im->width){maxx=im->width;}
	if(miny<0){miny=0;}
	if(maxy>im->height){maxy=im->height;}

	CV_INIT_PIXEL_POS(pos_im,(unsigned char *) im->imageData,im->widthStep,cvGetSize(im),x,y,im->origin);

	for( y=miny;y<maxy; y++){
		for ( x=minx; x<maxx; x++){
			ptr_im = CV_MOVE_TO(pos_im,x,y,3);
			ptr_im[0] = value;
			ptr_im[1] = value;
			ptr_im[2] = value;
		}
	}
}

void COpenCvImage::removeNoise(){
	IplImage* pyr;
	CvSize sz = cvSize( im->width & -2, im->height & -2);
	pyr = cvCreateImage( cvSize(sz.width/2, sz.height/2), 8,3 );
	cvPyrDown( im, pyr, 7 );
	//remove noise from input
	cvPyrUp( pyr, im, 7 );
}

bool COpenCvImage::testMaskZone(double pourcentage,CvRect zone){
	return testMaskZone(pourcentage,zone.x,zone.y,zone.x+zone.width,zone.y+zone.height);
}

bool COpenCvImage::testMaskZone(double pourcentage,int minx,int miny,int maxx,int maxy){
	uchar * ptr_skinMask;
	int x=0,y=0;
	int nbrPixelOn=0;
	int nbrPixel=0;

	if(minx<0){minx=0;}
	if(maxx>im->width){maxx=im->width;}
	if(miny<0){miny=0;}
	if(maxy>im->height){maxy=im->height;}

	for( y=miny;y<maxy; y++){
		for ( x=minx; x<maxx; x++){
			ptr_skinMask = cvMoveTo(x,y);
			if( ptr_skinMask[0] == 255 ){
				nbrPixelOn++;
			}
			nbrPixel++;
		}
	}
	return pourcentage/100.0<(double)nbrPixelOn/(double)(nbrPixel);
}

/*
void COpenCvImage::deltaColorMask(COpenCvImage * src_RGB,CDeltaColor deltaColor){
	uchar Y,Cr,Cb;
	uchar * ptr_src;
	uchar * ptr_dst;
	int x =0;
	int y =0;

	int maxy=src_RGB->height();
	int maxx=src_RGB->width();
	for( y=0;y<maxy; y++){
		for ( x=0; x<maxx; x++){
			ptr_src = src_RGB->cvMoveTo(x,y);
			ptr_dst = this->cvMoveTo(x,y);
			Y = ptr_src[0];
			Cb= ptr_src[1];
			Cr= ptr_src[2];

			if( deltaColor.test(Y,Cr,Cb)){
				ptr_dst[0] = 255;
				ptr_dst[1] = 255;
				ptr_dst[2] = 255;
			}else{
				ptr_dst[0] = 0;
				ptr_dst[1] = 0;
				ptr_dst[2] = 0;
			}
		}
	}
}*/

void COpenCvImage::deltaColorMask(COpenCvImage * src_RGB,CDeltaColor deltaColor){
	COpenCvImage src(src_RGB);
	COpenCvImage tmpYCR(src_RGB->size());
	//COpenCvImage * dst=new COpenCvImage(src_RGB->size());
	uchar Y,Cr,Cb;
	uchar * ptr_src;
	uchar * ptr_dst;
	int x =0;
	int y =0;

	src.removeNoise();
	cvCvtColor(src.im ,tmpYCR.im , CV_RGB2YCrCb);

	int maxy=src.height();
	int maxx=src.width();
	for( y=0;y<maxy; y++){
		for ( x=0; x<maxx; x++){
			ptr_src = tmpYCR.cvMoveTo(x,y);
			ptr_dst = this->cvMoveTo(x,y);
			Y = ptr_src[0];
			Cb= ptr_src[1];
			Cr= ptr_src[2];

			if( deltaColor.test(Y,Cr,Cb,80)){
				ptr_dst[0] = 255;
				ptr_dst[1] = 255;
				ptr_dst[2] = 255;
			}else{
				ptr_dst[0] = 0;
				ptr_dst[1] = 0;
				ptr_dst[2] = 0;
			}
		}
	}
}


void COpenCvImage::skinMask(COpenCvImage * src_RGB,CDeltaColor dc){
	COpenCvImage src(src_RGB);
	COpenCvImage tmpYCR(src_RGB->size());
	uchar Y,Cr,Cb;
	uchar * ptr_src;
	uchar * ptr_dst;
	int x =0;
	int y =0;

	src.removeNoise();

	cvCvtColor(src.im ,tmpYCR.im , CV_RGB2YCrCb);

	int maxy=src.height();
	int maxx=src.width();
	for( y=0;y<maxy; y++){
		for ( x=0; x<maxx; x++){
			ptr_src = tmpYCR.cvMoveTo(x,y);
			ptr_dst = this->cvMoveTo(x,y);
			Y = ptr_src[0];
			Cb= ptr_src[1];
			Cr= ptr_src[2];

			if(CYCrCb::testSkinColor(Cr,Cb,dc.average.Cr,5)){
				ptr_dst[0] = 255;
				ptr_dst[1] = 255;
				ptr_dst[2] = 255;
			}else{
				ptr_dst[0] = 0;
				ptr_dst[1] = 0;
				ptr_dst[2] = 0;
			}
		}
	}
	//printf("dc.average.Cr%d\n",dc.average.Cr);
	//dilate(1);
	//erode(4);
}
/*
void COpenCvImage::skinMask(COpenCvImage * src_RGB,CDeltaColor dc){
	COpenCvImage src(src_RGB);
	COpenCvImage tmpYCR(src_RGB->size());
	uchar Y,Cr,Cb;
	uchar * ptr_src;
	uchar * ptr_dst;
	int x =0;
	int y =0;

	src.removeNoise();

	cvCvtColor(src.im ,tmpYCR.im , CV_RGB2YCrCb);

	int maxy=src.height();
	int maxx=src.width();
	for( y=0;y<maxy; y++){
		for ( x=0; x<maxx; x++){
			ptr_src = tmpYCR.cvMoveTo(x,y);
			ptr_dst = this->cvMoveTo(x,y);
			Y = ptr_src[0];
			Cb= ptr_src[1];
			Cr= ptr_src[2];

			if(CYCrCb::testSkinColor(Y,Cr,Cb,dc.average.Cr,5)){
				ptr_dst[0] = 255;
				ptr_dst[1] = 255;
				ptr_dst[2] = 255;
			}else{
				ptr_dst[0] = 0;
				ptr_dst[1] = 0;
				ptr_dst[2] = 0;
			}
		}
	}
	//printf("dc.average.Cr%d\n",dc.average.Cr);
	//dilate(1);
	//erode(4);
}
*/

void COpenCvImage::skinMask(COpenCvImage * src_RGB){
	COpenCvImage src(src_RGB);
	COpenCvImage tmpYCR(src_RGB->size());
	//COpenCvImage * dst=new COpenCvImage(src_RGB->size());
	uchar Y,Cr,Cb;
	uchar * ptr_src;
	uchar * ptr_dst;
	int x =0;
	int y =0;

	src.removeNoise();

	cvCvtColor(src.im ,tmpYCR.im , CV_RGB2YCrCb);

	int maxy=src.height();
	int maxx=src.width();
	for( y=0;y<maxy; y++){
		for ( x=0; x<maxx; x++){
			ptr_src = tmpYCR.cvMoveTo(x,y);
			ptr_dst = this->cvMoveTo(x,y);
			Y = ptr_src[0];
			Cb= ptr_src[1];
			Cr= ptr_src[2];

			if(CYCrCb::testSkinColor(Cr,Cb)){
				ptr_dst[0] = 255;
				ptr_dst[1] = 255;
				ptr_dst[2] = 255;
			}else{
				ptr_dst[0] = 0;
				ptr_dst[1] = 0;
				ptr_dst[2] = 0;
			}
		}
	}
}

COpenCvImage * COpenCvImage::getSkinMask(COpenCvImage * src_RGB){
	COpenCvImage src(src_RGB);
	COpenCvImage tmpYCR(src_RGB->size());
	COpenCvImage * dst=new COpenCvImage(src_RGB->size());
	uchar Y,Cr,Cb;
	uchar * ptr_src;
	uchar * ptr_dst;
	int x =0;
	int y =0;

	src.removeNoise();

	cvCvtColor(src.im ,tmpYCR.im , CV_RGB2YCrCb);

	int maxy=src.height();
	int maxx=src.width();
	for( y=0;y<maxy; y++){
		for ( x=0; x<maxx; x++){
			ptr_src = tmpYCR.cvMoveTo(x,y);
			ptr_dst = dst->cvMoveTo(x,y);
			Y = ptr_src[0];
			Cb= ptr_src[1];
			Cr= ptr_src[2];

			if( CYCrCb::testSkinColor(Cr,Cb)){
				ptr_dst[0] = 255;
				ptr_dst[1] = 255;
				ptr_dst[2] = 255;
			}else{
				ptr_dst[0] = 0;
				ptr_dst[1] = 0;
				ptr_dst[2] = 0;
			}
		}
	}

	//if(erosions>0) cvErode(dst->im,dst->im,0,erosions);
	//if (dilations>0) cvDilate(dst->im,dst->im,0,dilations);
	return dst;
}


void COpenCvImage::mvtMask(COpenCvImage * srcRGBOld, COpenCvImage * srcRGBNew, int threshold){
	uchar YOld,YNew;
	//uchar YOld,CrOld,CbOld,YNew,CrNew,CbNew;
	uchar * ptr_srcOld;
	uchar * ptr_srcNew;
	uchar * ptr_dst;
/*
	COpenCvImage srcRGBOld(psrcRGBOld);
	COpenCvImage srcRGBNew(psrcRGBNew);
	int ymax=srcRGBNew.height();
	int xmax=srcRGBNew.width();
*/
	int ymax=srcRGBNew->height();
	int xmax=srcRGBNew->width();

	int y=0;int x=0;
	//printf("GetMvtImage : threshold=%d ymax=%d xmax=%d",threshold,ymax,xmax);
	for(y=0;y<ymax; y++){
		for (x=0; x<xmax; x++){
			ptr_srcOld = srcRGBOld->cvMoveTo(x,y);
			ptr_srcNew = srcRGBNew->cvMoveTo(x,y);
			ptr_dst = this->cvMoveTo(x,y);
			
			YOld = ptr_srcOld[0];
			//CbOld= ptr_src_old[1];
			//CrOld= ptr_src_old[2];
			YNew = ptr_srcNew[0];
			//CbNew= ptr_src_new[1];
			//CrNew= ptr_src_new[2];
			//printf("YOld=%d YNew=%d",YOld,YNew);
			if( YOld-YNew > threshold || YOld-YNew < -threshold ){
				ptr_dst[0] = 255;
				ptr_dst[1] = 255;
				ptr_dst[2] = 255;
			}else{
				ptr_dst[0] = 0;
				ptr_dst[1] = 0;
				ptr_dst[2] = 0;
			}
		}
		//printf(".");
	}
	//printf("done\n");
}

COpenCvImage * COpenCvImage::getMvtImage(COpenCvImage * srcRGBOld, COpenCvImage * srcRGBNew, int threshold){
	COpenCvImage * dst=new COpenCvImage(srcRGBOld->size());

	uchar YOld,YNew;
	//uchar YOld,CrOld,CbOld,YNew,CrNew,CbNew;
	uchar * ptr_srcOld;
	uchar * ptr_srcNew;
	uchar * ptr_dst;

	//COpenCvImage srcRGBOld(psrcRGBOld);
	//COpenCvImage srcRGBNew(psrcRGBNew);

	//int ymax=srcRGBNew.height();
	//int xmax=srcRGBNew.width();

	int ymax=srcRGBNew->height();
	int xmax=srcRGBNew->width();
	int y=0;int x=0;
	//printf("GetMvtImage : threshold=%d ymax=%d xmax=%d",threshold,ymax,xmax);
	for(y=0;y<ymax; y++){
		for (x=0; x<xmax; x++){
			ptr_srcOld = srcRGBOld->cvMoveTo(x,y);
			ptr_srcNew = srcRGBNew->cvMoveTo(x,y);
			ptr_dst = dst->cvMoveTo(x,y);
			
			YOld = ptr_srcOld[0];
			//CbOld= ptr_src_old[1];
			//CrOld= ptr_src_old[2];
			YNew = ptr_srcNew[0];
			//CbNew= ptr_src_new[1];
			//CrNew= ptr_src_new[2];
			//printf("YOld=%d YNew=%d",YOld,YNew);
			if( YOld-YNew > threshold || YOld-YNew < -threshold ){
				ptr_dst[0] = 255;
				ptr_dst[1] = 255;
				ptr_dst[2] = 255;
			}else{
				ptr_dst[0] = 0;
				ptr_dst[1] = 0;
				ptr_dst[2] = 0;
			}
		}
		
		//printf(".");
	}
	//printf("done\n");
	return dst;
}