// PeopleWithHands.cpp: implementation of the CPeopleWithHands class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "PeopleWithHands.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPeopleWithHands::CPeopleWithHands(CvRect * head):m_openCvTracker(0){
	m_head.x=head->x;
	m_head.y=head->y;
	m_head.width=head->width;
	m_head.height=head->height;
	resetLR();
}

CPeopleWithHands::~CPeopleWithHands(){
	if(m_openCvTracker){delete m_openCvTracker;}
}

int CPeopleWithHands::distance(CvRect * r){
	int dx = r->x+r->width/2-(m_head.x+m_head.width/2);
	int dy = r->y+r->height/2-(m_head.y+m_head.height/2);
	return (dx*dx)+(dy*dy);
}

CvRect CPeopleWithHands::getCloserRightHand(CSkinMask * skinMask){
	CCvRectList cvRectList;
	cvRectList.addCvRectsFromMask(skinMask,m_rightZone,50,m_rightHand.width,m_rightHand.height);
	return cvRectList.getCloserCvRect(m_head.x,m_head.y+m_head.height/2);
}
CvRect CPeopleWithHands::getCloserLeftHand(CSkinMask * skinMask){
	CCvRectList cvRectList;
	cvRectList.addCvRectsFromMask(skinMask,m_leftZone,50,m_leftHand.width,m_leftHand.height);
	return cvRectList.getCloserCvRect(m_head.x+m_head.width,m_head.y+m_head.height/2);
}

void CPeopleWithHands::setCloserHand(CCvRectList & cvRectList){
	int rx=m_head.x;
	int ry=m_head.y+m_head.height/2;
	int lx=m_head.x+m_head.width;
	int ly=m_head.y+m_head.height/2;

	CvRect r=cvRectList.getCloserCvRect(rx,ry);
	CvRect l=cvRectList.getCloserCvRect(lx,ly);
/*
	if(r.x!=-1&&l.x!=-1){
		bool inRightZone=CCommun::insideTest(m_rightZone,r);
		bool inLeftZone=CCommun::insideTest(m_leftZone,l);

	}*/
	if(r.x!=-1&&l.x!=-1){
		if(((r.x-rx)*(r.x-rx)+(r.y-ry)*(r.y-ry))>((l.x-lx)*(l.x-lx)+(l.y-ly)*(l.y-ly))){
			m_leftHand.x=l.x;
			m_leftHand.y=l.y;
			cvRectList.remove(l);
		}else{
			m_rightHand.x=r.x;
			m_rightHand.y=r.y;
			cvRectList.remove(r);
		}
	}
}

bool CPeopleWithHands::getVote(){
	return m_rightHand.x!=-1||m_leftHand.x!=-1;
}

CvRect CPeopleWithHands::getBodyCvRect(){
	CvRect body;
	body.x=(int)(m_head.x-1.5*m_head.width);
	body.y=m_head.y+m_head.height;
	body.width=4*m_head.width;
	body.height=2*m_head.height;
	return body;
}

bool CPeopleWithHands::updateHead(CvRect * head){
	if(isTheSame(head)){
		m_head.x=head->x;
		m_head.y=head->y;
		m_head.width=head->width;
		m_head.height=head->height;
		return true;
	}
	return false;
}

bool CPeopleWithHands::isTheSame(CvRect * r){
	int dx = r->x+r->width/2-(m_head.x+m_head.width/2);
	int dy = r->y+r->height/2-(m_head.y+m_head.height/2);
	return dx*dx<m_head.width*m_head.width&&dy*dy<m_head.height*m_head.height;
	//return distance(r)<m_head.width*m_head.width+m_head.height*m_head.height;
}

void CPeopleWithHands::drawHead(COpenCvImage* frame,CvScalar color){
	frame->drawCvRect(&m_head,color);
}

void CPeopleWithHands::draw(COpenCvImage* frame, CvScalar color){
	drawHead(frame,color);
	if(m_leftHand.x!=-1){
		frame->drawCvRect(&m_leftHand,color);
	}
	if(m_rightHand.x!=-1){
		frame->drawCvRect(&m_rightHand,color);
	}
}

void CPeopleWithHands::autoTrackingWithSkinMask(COpenCvImage * skinMaskOld,COpenCvImage * skinMask){
	if(m_openCvTracker){
		m_openCvTracker->track(skinMask,&m_head);
	}else{
		m_openCvTracker=new COpenCvTracker(skinMask,m_head);
	}
}

bool CPeopleWithHands::isInHandZone(CvRect & hand){
	return CCommun::insideTest(m_leftZone,hand)||CCommun::insideTest(m_rightZone,hand);
}

void CPeopleWithHands::resetLR(){
	m_leftHand.x=-1;
	m_leftHand.y=-1;
	m_leftHand.width=(int)(m_head.width/2);
	m_leftHand.height=(int)(m_head.height/2);

	m_rightHand.x=-1;
	m_rightHand.y=-1;
	m_rightHand.width=(int)(m_head.width/2);
	m_rightHand.height=(int)(m_head.height/2);
}

void CPeopleWithHands::updateRLZone(COpenCvImage* frame){
	int minx,miny,maxx,maxy;

	int fw=frame->width();
	int fh=frame->height();

	minx = (int)(m_head.x-1.5*m_head.width);
	maxx = m_head.x;
	miny = m_head.y-m_head.height;
	maxy = m_head.y+m_head.height;
	if(minx<0){minx=0;}
	if(maxx>fw){maxx=fw;}
	if(miny<0){miny=0;}
	if(maxy>fh){maxy=fh;}
	m_rightZone.x=minx;
	m_rightZone.y=miny;
	m_rightZone.width=maxx-minx;
	m_rightZone.height=maxy-miny;

	minx = m_head.x+m_head.width;
	maxx = (int)(m_head.x+2.5*m_head.width);
	miny = m_head.y-m_head.height;
	maxy = m_head.y+m_head.height;
	if(minx<0){minx=0;}
	if(maxx>fw){maxx=fw;}
	if(miny<0){miny=0;}
	if(maxy>fh){maxy=fh;}
	m_leftZone.x=minx;
	m_leftZone.y=miny;
	m_leftZone.width=maxx-minx;
	m_leftZone.height=maxy-miny;
}

void CPeopleWithHands::stupidHandsDetection(CSkinMask* skinMask){
	resetLR();
	CvRect rightHand=getCloserRightHand(skinMask);
	if(rightHand.x!=-1){
		m_rightHand.x=rightHand.x;
		m_rightHand.y=rightHand.y;
		skinMask->removeCvRect(&m_rightHand);
	}
	CvRect leftHand=getCloserLeftHand(skinMask);
	if(leftHand.x!=-1){
		m_leftHand.x=leftHand.x;
		m_leftHand.y=leftHand.y;
		skinMask->removeCvRect(&m_leftHand);
	}
	/*
	CCvPointList cvPointList;
	cvPointList.getAveragePoints(skinMask,m_rightZone.x,m_rightZone.y,m_rightZone.x+m_rightZone.width,m_rightZone.y+m_rightZone.height,70,m_rightHand.height,m_rightHand.width);
	CvPoint rightHand=cvPointList.getCloserCvPoint(m_head.x,m_head.y+m_head.height/2);
	if(rightHand.x!=-1){
		m_rightHand.x=rightHand.x-m_rightHand.width/2;
		m_rightHand.y=rightHand.y-m_rightHand.height/2;
	}
	CCvPointList cvPointList2;
	cvPointList2.getAveragePoints(skinMask,m_leftZone.x,m_leftZone.y,m_leftZone.x+m_leftZone.width,m_leftZone.y+m_leftZone.height,70,m_leftHand.height,m_leftHand.width);
	CvPoint leftHand=cvPointList2.getCloserCvPoint(m_head.x+m_head.width,m_head.y+m_head.height/2);
	if(leftHand.x!=-1){
		m_leftHand.x=leftHand.x-m_leftHand.width/2;
		m_leftHand.y=leftHand.y-m_leftHand.height/2;
	}*/
}