#include "StdAfx.h"
#include ".\skinmask.h"

CSkinMask::CSkinMask(COpenCvImage * original,int erosion,int dilatation):COpenCvImage(original->size()){
	update(original);
}

CSkinMask::~CSkinMask(void){}

void CSkinMask::update(COpenCvImage * src_RGB){
	COpenCvImage src(src_RGB);
	COpenCvImage tmpYCR(src_RGB->size());
	uchar Y,Cr,Cb;
	uchar * ptr_src;
	uchar * ptr_im;
	int x =0;
	int y =0;

	src.removeNoise();

	cvCvtColor(src.im ,tmpYCR.im , CV_RGB2YCrCb);

	int maxy=src.height();
	int maxx=src.width();
	for( y=0;y<maxy; y++){
		for ( x=0; x<maxx; x++){
			ptr_src = tmpYCR.cvMoveTo(x,y);
			ptr_im = cvMoveTo(x,y);
			Y = ptr_src[0];
			Cb= ptr_src[1];
			Cr= ptr_src[2];

			if( CYCrCb::testSkinColor(Cr,Cb)){
				ptr_im[0] = 255;
				ptr_im[1] = 255;
				ptr_im[2] = 255;
			}else{
				ptr_im[0] = 0;
				ptr_im[1] = 0;
				ptr_im[2] = 0;
			}
		}
	}
}
 
void CSkinMask::update2(COpenCvImage * original){
    // Threshold vars
    /*int rmin = 38;
    int rmax = 43;
    int gmin = 30;
    int gmax = 34;*/
	int rmin = 38;
    int rmax = 43;
    int gmin = 30;
    int gmax = 34;
	detectSkin(original->im, rmin/100.0, rmax/100.0, gmin/100.0, gmax/100.0);
}

void CSkinMask::detectSkin(IplImage* img, double rmin, double rmax, double gmin, double gmax)
{
    // Split image into individual channels
    IplImage* img32F = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F, 3);
    cvConvertScale(img, img32F, 1, 0);
    IplImage* imgRed = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F, 1);
    IplImage* imgGreen = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F, 1);
    IplImage* imgBlue = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F, 1);
    cvSplit(img32F, imgBlue, imgGreen, imgRed, NULL);
 
    // Create intensity map
    IplImage* intensity = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F, 1);
    cvAdd(imgRed, imgGreen, intensity);
    cvAdd(intensity, imgBlue, intensity);
    
    // Calculate rgb ratios
    cvDiv(imgRed, intensity, imgRed, 256);
    cvDiv(imgGreen, intensity, imgGreen, 256);
    //cvDiv(imgBlue, intensity, imgBlue, 256);        // unnecessary
 
    // Create skin mask
    IplImage* rskin = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1);
    IplImage* gskin = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1);
    IplImage* skin = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1);
    cvInRangeS(imgRed, cvRealScalar(256 * rmin), cvRealScalar(256 * rmax), rskin);
    cvInRangeS(imgGreen, cvRealScalar(256 * gmin), cvRealScalar(256 * gmax), gskin);
    cvAnd(rskin, gskin, skin);
   
    // Filter out noise
    cvErode(skin, skin, NULL, 2);
    cvDilate(skin, skin, NULL, 2);
    cvMorphologyEx(skin, skin, NULL, NULL, CV_MOP_OPEN);
    cvMorphologyEx(skin, skin, NULL, NULL, CV_MOP_CLOSE);
    
    //IplImage* skinmask = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 3);
    cvMerge(skin, skin, skin, NULL, im);
 
    // Release images
    cvReleaseImage(&img32F);
    cvReleaseImage(&intensity);
    cvReleaseImage(&imgRed);
    cvReleaseImage(&imgGreen);
    cvReleaseImage(&imgBlue);
    cvReleaseImage(&rskin);
    cvReleaseImage(&gskin);
    cvReleaseImage(&skin);
    
    //return skinmask;
}
 
